@@ -1,3 +1,4 @@
+import sys
 import pytest
 from mando import Program
 
@@ -32,7 +33,9 @@ def test_generic_command(args, result):
     assert result == program.execute(args)
     assert program.parse(args)[0].__name__ == program._current_command
 
-
+ending = 'al arguments'
+if sys.version_info[:2] >= (3, 10):
+    ending = 's'
 GOOGLE_DOCSTRING_HELP_CASES = [
     ('simple_google_docstring --help 2 --arg2=test', '''usage: example.py simple_google_docstring [-h] [--arg2 ARG2] arg1
 
@@ -41,10 +44,10 @@ Extended description.
 positional arguments:
   arg1         Description of `arg1`
 
-optional arguments:
+option%s:
   -h, --help   show this help message and exit
   --arg2 ARG2  Description of `arg2`
-'''),
+''' % ending),
 ]
 
 
